require 'jwt'
require 'warden'

CONTENT_TYPE = { "Content-Type" => "application/json" }
GOOD_GUYS    = { "a" => "1", "b" => "2" }
SECRET       = "secret"

failure_app = Proc.new { |env| ['401', CONTENT_TYPE, ["UNAUTHORIZED"]] }

Warden::Strategies.add(:jwt_token) do
  def valid?
    !jwt_token.nil?
  end

  def authenticate!
    user = JWT.decode(jwt_token, SECRET)
    user.nil? ? fail("!") : success!(user[0])
  end

  def jwt_token
    env['HTTP_AUTHORIZATION'].match(/JWT token="(?<token>.+)"/)[:token]
  end
end

Warden::Strategies.add(:logpass) do
  def valid?
    params["login"] && params["password"]
  end

  def authenticate!
    env["password_auth"] = true
    password = GOOD_GUYS[params["login"]]
    password_compare = secure_compare password, params["password"]
    password_compare ? success!(params["login"]) : fail("Invalid credentials")
  end

  def secure_compare(a, b)
    return false if a == "" || b == "" || a.bytesize != b.bytesize
    l = a.unpack "C#{a.bytesize}"

    res = 0
    b.each_byte { |byte| res |= byte ^ l.shift }
    res == 0
  end
end

class Application
  def self.call(env)
    env['warden'].authenticate!
    if env['password_auth']
      return [200, CONTENT_TYPE, [JWT.encode({user: env["warden"].user}, SECRET)] ]
    end

    [200, CONTENT_TYPE, [env["warden"].user.to_json] ]
  end
end

run Rack::Builder.new do
  use Rack::Session::Cookie, :secret => "replace this with some secret key"
   use Warden::Manager do |manager|
    manager.default_strategies :logpass,:jwt_token
    manager.failure_app = failure_app
  end
  run Application
end

# curl -H 'Authorization: JWT token="eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1c2VyIjoiYSJ9.DmNqJom5ipZleBZKpGi3pnHHHTkVsx1MwzdJ84sLQoE"' localhost:9292
# curl localhost:9292?login=a\&password=1
